package com.example.demo;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.util.retry.Retry;

import java.time.Duration;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

public class FluxErrorAndOtherMethods {

    public Flux<String> stringFlux() {
        return 	Flux.just("These","Strings","will","create","flux").log();
    }

    public Flux<String> stringFluxWithconcat() {
        return Flux.just("These","Strings","will","create","flux")
                .concatWith(Flux.just("new message")).log();
    }

    public Flux<String> stringFluxSorting() {
        return Flux.just("These","Strings","will","create","flux")
                .sort(Comparator.reverseOrder()).log();
    }

    public Flux<String> stringFluxDistinct() {
        return Flux.just("These","These","Strings","Strings","will","will","create","flux")
                .distinct()
                .log();
    }

    public Flux<String> stringFluxFilter() {
        return Flux.just("Ravi", "Ravendra", "Manoj", "Manish", "Mounika", "Devi", "Sandhya", "Prakash", "Aravind", "Basha", "Chaitu" )
                .filter( s -> s.length() <= 5 ).log();
    }

    public Flux<String> stringFluxMap() {
        return Flux.just("These","Strings","will","create","flux")
                .map( s -> s.toUpperCase() ).log();
    }

    public Flux<String> stringFluxFlatMap() {
        return Flux.just("These","Strings","will","create","flux")
                .flatMap( s -> Flux.just(s.concat("-@1")) ).log();
    }

    public Flux<Integer> fluxOnErrorReturn() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .log()
                .onErrorReturn( 100 ).log();
    }


    public Flux<Integer> fluxOnErrorReturnWithClassType() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .log()
                .onErrorReturn( RuntimeException.class, 100 ).log();
    }

    public Flux<Integer> fluxOnErrorResume() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .onErrorResume( RuntimeException.class, ex -> Flux.range( 1, 3 ) ).log();
    }

    public Flux<Integer> fluxOnErrorMap() {
/*
        Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .onErrorMap( RuntimeException.class, ex -> new RuntimeException("we messed up somewhere") ).log();
*/
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .onErrorMap( ex -> new RuntimeException("we messed up somewhere") ).log();
    }

    public Flux<Integer> fluxDoOnError() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .doOnError( ex -> new RuntimeException("we messed up somewhere") )
                .log();
    }

    public Flux<Integer> fluxRetry() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .retry(3)
                .log();
    }

    public Flux<Integer> fluxOnRetryWhen() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .retryWhen(Retry.max(3))
                .log();
    }

    public Flux<Integer> fluxOnRetryWhenBackOff() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .retryWhen(Retry.backoff(2, Duration.ofSeconds(2)))
                .log();
    }

    public Flux<Integer> fluxOnRetryWhenFixedDelay() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .retryWhen(Retry.fixedDelay(2, Duration.ofSeconds(5)))
                .log();
    }

    public Mono<List<Integer>> fluxCollectList() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .collectList().log();
    }

    public Mono<List<Integer>> fluxCollectSortedList() {
        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .collectSortedList( Comparator.reverseOrder() ).log();
    }

    public Mono<Map<String, String>> fluxCollectMap() {
        return Flux.just( "krishna:8125486447", "ramu:8125719324", "balla:9908263541" )
                .collectMap( s -> s.split(":")[0],
                             s -> s.split(":")[1] ).log();
    }


    public static void fluxBlockFirst() {
        Flux.just( 1, 4, 5, 2, 0, 10 )
                .delayElements( Duration.ofSeconds(5)).log()
                .blockFirst( Duration.ofSeconds(10) );
    }

    public static void fluxBlockLast() {
        Flux.just( 1, 4, 5, 2, 0, 10 )
                .delayElements( Duration.ofSeconds(5)).log()
                .blockLast( Duration.ofSeconds(300) );
    }


    public Flux<Integer> fluxOnErrorComplete() {

        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .onErrorComplete( RuntimeException.class ).log();
    }

    public Flux<Integer> fluxOnErrorContinue() {

        return Flux.just( 1, 4, 5, 2, 0, 10 )
                .map( x -> 100/x )
                .onErrorContinue( RuntimeException.class, (throwable, o) -> System.out.println( "Exception: "+ throwable.toString() ) ).log();
    }


    public static void main(String[] args) {
       // fluxBlockFirst();
        fluxBlockLast();
    }

}
